@*****************************************************************************@
@                                                                             @
@                              D . P . D .  98                                @
@                                                                             @
@        A program in the GAUSS matrix programming language to compute        @
@        GMM estimates for a dynamic model from unbalanced panel data.        @
@                                                                             @
@        Manuel Arellano and Stephen Bond                     Dec 1998        @
@                                                                             @
@*****************************************************************************@

output off; cls; locate 1,1;

"              DDDDDDDDD        PPPPPPPPPP       DDDDDDDDD                   ";
"              DDD    DDD       PPP     PPP      DDD    DDD                  ";
"              DDD     DDD      PPP      PPP     DDD     DDD                 ";
"              DDD      DDD     PPP     PPP      DDD      DDD       999 888  ";
"              DDD      DDD     PPPPPPPPPP       DDD      DDD       9 9  8   ";
"              DDD     DDD      PPP              DDD     DDD        999 888  ";
"              DDD    DDD       PPP              DDD    DDD           9  8   ";
"              DDDDDDDDD    .   PPP          .   DDDDDDDDD     .      9 888  ";

?;
"                                                          Version 30/12/98";

iiv=1; ijus=0; format /m1 /rd 1,0;

@---------------------------- repeated comparisons ---------------------------@

let v01=0 1;
let v34=3 4;
let m034=0 3 4;
let m0125=0 1 2 5;
let m34=3 4;
let m25=2 5;
let m4=4;
let m125=1 2 5;
let c125=1 2 5;
let c245=2 4 5;
let c45=4 5;
let c15=1 5;
let c25=2 5;

@--------------------------- prompt user for set up --------------------------@

if bat/=1;

inval=0;
check1:
locate 15,1; if inval==1; clin=cline(11); endif; locate 15,1;
" State form of model";
" - type 0 for levels";
"        1 for first differences";
"        2 for orthogonal deviations";
"        3 for combined first differences and levels";
"        4 for combined orthogonal deviations and levels";
"        5 for within groups";
imod=con(1,1);

if minc(imod./=seqa(0,1,9));
  ?; inval=1;
  " ******************  Invalid option. Please reselect.  ****************** ";
  goto check1;
elseif inval==1; clin=cline(2); inval=0;
endif;

m034=maxc(imod.==m034);
m0125=maxc(imod.==m0125);
m34=maxc(imod.==m34);

check2:
if inval==0; locate 12,1; clin=cline(11);
locate 13,5; "model = ";; imod;; else; locate 15,1;
if m034; clin=cline(8); else; clin=cline(5); endif; endif;
locate 15,1;
" Select from choice of constants to be included";
" - type 0 for none";
"        1 for time dummies only";
"        2 for time dummies interacted with industry dummies";
if m034;
"        3 for constant only";
"        4 for industry dummies only";
"        5 for time dummies and industry dummies"; endif;
icon=con(1,1);

if minc(icon./=seqa(0,1,6));
  ?; inval=1;
  " ******************  Invalid option. Please reselect.  ****************** ";
  goto check2;
elseif inval==1; clin=cline(2); inval=0;
endif;

check3:
if inval==0; locate 15,1;
if m034; clin=cline(8); else; clin=cline(5); endif;
locate 13,16; "const = ";; icon;;
else; locate 16,1; clin=cline(3); endif; locate 16,1;

if m0125;

" Do you want robust test statistics and two-step estimates";
" - type 1 for yes or 0 for no";
irob=con(1,1);

if minc(irob./=v01);
  ?; inval=1;
  " ******************  Invalid option. Please reselect.  ****************** ";
  goto check3;
elseif inval==1; clin=cline(2); inval=0;
endif;

endif;

if m34;
  irob=1;
endif;

check4:
locate 16,1; if inval==0; clin=cline(3);
locate 13,27; "robust = ";; irob;; else; clin=cline(2); endif;
locate 16,1;
" Print descriptive statistics {1/0}"; ides=con(1,1);

if minc(ides./=v01);
  ?; inval=1;
  " ******************  Invalid option. Please reselect.  ****************** ";
  goto check4;
elseif inval==1; clin=cline(2); inval=0;
endif;

check5:
locate 16,1; clin=cline(2);
if inval==0; locate 13,39; "desc = ";; ides;; endif; locate 16,1;
" Print parameter covariance matrices {1/0}"; icov=con(1,1);

if minc(icov./=v01);
  ?; inval=1;
  " ******************  Invalid option. Please reselect.  ****************** ";
  goto check5;
elseif inval==1; clin=cline(2); inval=0;
endif;

check6:
locate 16,1; clin=cline(2);
if inval==0; locate 13,49; "covar = ";; icov;; endif; locate 16,1;
" Save results to Gauss matrices {1/0}"; isav=con(1,1);

if minc(isav./=v01);
  ?; inval=1;
  " ******************  Invalid option. Please reselect.  ****************** ";
  goto check6;
elseif inval==1; clin=cline(2); inval=0;
endif;

locate 13,60; "save = ";; isav;;

else;

m0125=maxc(imod.==m0125);
m34=maxc(imod.==m34);

if m34;
  irob=1;
endif;

locate 13,5; "model = ";; imod;; locate 13,16; "const = ";; icon;;
locate 13,27; "robust = ";; irob;; locate 13,39; "desc = ";; ides;;
locate 13,49; "covar = ";; icov;; locate 13,60; "save = ";; isav;;

endif;

m25=maxc(imod.==m25);
m125=maxc(imod.==m125);
mn01=minc(imod./=v01);
mn34=minc(imod./=v34);
m0=imod==0;
m1=imod==1;
m3=imod==3;
m4=imod==4;
mn1=imod/=1;
c125=maxc(icon.==c125);
c245=maxc(icon.==c245);
c45=maxc(icon.==c45);
c15=maxc(icon.==c15);
c25=maxc(icon.==c25);
c2=icon==2;
c4=icon==4;
c5=icon==5;
cn0=icon/=0;
d1=ides==1;
wald1=waldtest==1;

@----------------------------- begin computations ----------------------------@

speed=hsec;

let tl=1 2 5;
tl=maxc(imod.==tl);           @ tl=no of equations lost due to transformation @

ts=imod/=0;            @ ts=no of residuals lost for serial correlation tests @

inf1=readr(f2,stopf2);            @ read auxiliary data file and make startf1 @
if startf2==1;
  startf1=1;
else;
  startf1=inf1[1:startf2-1,1]'inf1[1:startf2-1,2]+1;
endif;

inf1=inf1[startf2:stopf2,.];

ra=int(inf1[1,2]/ncomp);      @ make inf to control reading of main data file @
rb=inf1[1,2]%ncomp;
if ra<1; inf=inf1[1,.]; else; inf=inf1[1,1]~ncomp;
bj=2; do while bj<=ra; inf=inf|(inf1[1,1]~ncomp); bj=bj+1; endo;
if rb>0; inf=inf|(inf1[1,1]~rb); endif; endif;
aj=2; do while aj<=rows(inf1);
ra=int(inf1[aj,2]/ncomp); rb=inf1[aj,2]%ncomp;
if ra<1; inf=inf|inf1[aj,.]; else;
bj=1; do while bj<=ra; inf=inf|(inf1[aj,1]~ncomp); bj=bj+1; endo;
if rb>0; inf=inf|(inf1[aj,1]~rb); endif; endif;
aj=aj+1; endo;

clear inf1;
mm=rows(inf); infmax=maxc(inf); tmax=infmax[1,1];

sc1=tmax-lag-ts>=2;
sc2=tmax-lag-ts>=3;

dvd=seqa(1,1,nyears);
dv=dvd[lag+tl+1:nyears,.];
dv1=seqa(1,1,indmax);

ef=seekr(f1,startf1);
r=1;
goto done;

@---------------------------- subroutine defdata -----------------------------@

defdata:

  format 5,0;
  if mm<4900;
    stpr=5+floor((r-1)/70); stpc=5+(r-1)%70;
    if enpr>6 and stpc==5; locate 22,5;
  "                                                                       ";;
    locate 24,stpr; "*";; endif;
    locate 22,stpc; "*";;
    if r==1 and enpr>6; locate 24,enpr; "End";; endif;
    if stpr+1==enpr; locate 22,enpc; "End";; endif;
  else;
    locate 22,5; "Data blocks to read: ";; mm;;
  endif;
  locate 19,1;
  r;; t=inf[r,1]; t;; mul=inf[r,2]; mul;;

  if m34;                                      @ t1=observations on each firm @
    t1=2*(t-lag)-1;                                      @ used in estimation @
  else;
    t1=t-lag-tl;
  endif;

  t3=t-lag-ts;                                 @ t3=observations on each firm @
                                                @ in serial correlation tests @

  nr=mul*t;                                             @ make appropriate nr @
  data=readr(f1,nr);                                          @ read data set @

  hist=data[.,yearcol];
  dhist=hist-year1+1; fdums=dpdummy(dhist,dv);
  if m34; fzdums=fdums; endif;
  ind=data[.,indcol];
  indmiss=ismiss(ind);
  if indmiss==1;
    if c45;
    "*** Industry column contains missing values. Please respecify. ***";
    if sys==1; system; else; end; endif;
    else;
    ind=missrv(ind,0);
    endif;
  endif;
  idums=dpdummy(ind,dv1);
  clear y,x,z,sy,sx,ry,rx;

@---------------- data transformations and model specification ---------------@

  gosub model;

  clear zgmm,dtbi;

@------------------------- construct dummy variables -------------------------@

  if iiv==0; namez=namex; endif;

  if m0125; fdums=tlag(fdums,lag+tl,0,t);
  elseif m3; fdums=xcomfl(fdums,0,t); fzdums=xcomzl(fzdums,0,t);
  elseif m4; fdums=xcomol(fdums,0,t); fzdums=xcomzl(fzdums,0,t);
  endif;

  if c125; tdums=fdums[.,2:cols(fdums)];
    if m34; tzdums=fzdums[.,2:cols(fzdums)]; endif;
  endif;

  clear fzdums;

  if m0125; idums=tlag(idums,lag+tl,0,t);
  elseif m34; idums=xcomfl(idums,0,t);
  endif;

  if c45; indums=idums[.,2:cols(idums)]; endif;

  if c2;

    i=1;
    do while i<=mul;
      if i==1;
        ifdums=idums[t1,.].*.fdums[1:t1,.];
      else;
        ifdums=ifdums|idums[i*t1,.].*.fdums[(i-1)*t1+1:i*t1,.];
      endif;
    i=i+1; endo;

    itdums=ifdums[.,cols(fdums)+1:cols(ifdums)];

    clear ifdums;

  endif;

  clear fdums,idums,data;

@-----------------------------------------------------------------------------@

  if r==1; if cols(z)==cols(x) and iiv==1; ijus=1; endif; endif;

  kp=cols(x); namey=namey~namey;

  if d1;

    if m0125;
      yx=y~x;
    elseif m34;
      yx=rlev(y)~rlev(x);
    endif;

    nameyx=namex;
    if cols(nameyx)==1; nameyx=nameyx~nameyx; endif;
    kk=cols(yx);

  endif;

  if cn0;

    if m0125;
      const=ones(rows(y),1);
    elseif m3;
      const=xcomfl(ones(nr,1),0,t);
    elseif m4;
      const=xcomol(ones(nr,1),0,t);
    endif;

    x=const~x; namex="CONST"~namex; kp=kp+1;
    z=const~z; namez="CONST"~namez;

  endif;

  if c15;
    x=x~tdums; kd=kp+cols(tdums);
    if m34;
      z=z~tzdums; namez=namez~"TIM DUMS";
    else;
      z=z~tdums; namez=namez~"TIM DUMS";
    endif;
  endif;

  if c45;
    x=x~indums;
    z=z~indums; namez=namez~"IND DUMS";
  endif;

  if c2;
    x=x~tdums~itdums; kd=kp+cols(tdums);
    z=z~tdums~itdums; namez=namez~"INDUSTRY"~"TIM DUMS";
  endif;

  clear const,tdums,tzdums,indums,itdums;

  cinst=cols(z); if r==1; cinst;; endif;

  if cinst<cols(x);
    if sys==1; output on; endif;
    ?;?; " ***  More regressors than instruments. Please respecify model.";
    if sys==1; system; else; end; endif;
  endif;

  if m25;                                  @ use for serial correlation tests @
    sy=swap(y,t1); sx=swap(x,t1);
  elseif m3;
    sy=pick(y); sx=pick(x);
  elseif m4;
    sy=swap(pick(y),t-lag-1); sx=swap(pick(x),t-lag-1);
  else;
    sy=y; sx=x;
  endif;

  if m0125;                                                     @ use for RSS @
    ry=y; rx=x;
  elseif m34;
    ry=rlev(y); rx=rlev(x);
  endif;

  n=rows(ry);                           @ n=number of equations in this block @
  fms=fms+mul;                                          @ fms=number of firms @
  obs=obs+n;                                     @ obs=number of observations @
  k=cols(x);                              @ k=number of independent variables @

  if cols(namex)==1; namex=namex~namex; endif;

return;

@----------------------------- End of Subroutine -----------------------------@

done:

locate 16,1; clin=cline(2); locate 16,1;
"    **************************   Reading Data   **************************  ";
?;?;
if mm<4900; enpr=6+floor((mm-1)/70); enpc=6+(mm-1)%70; endif;
clear obs,sumry,xz,zy,yy,xx,xy,zkz,mom,sumyx,fms,y2y,x2x,y2x,x2y,y1y,x1x,y1x,
x1y,yky2,xkx2,ykx2,zky2,zkx2,yky1,xkx1,ykx1,zky1,zkx1,nc1,nc2,ryy,rxx,rxy;

@-------------------------------- first loop ---------------------------------@

do while r<=mm;

  gosub defdata;                                                  @ read data @

  yy=moment(y,0)+yy;                                   @ construct matrix y'y @
  xy=x'y+xy;                                           @ construct matrix x'y @
  xz=x'z+xz;                                           @ construct matrix x'z @
  zy=z'y+zy;                                           @ construct matrix z'y @
  xx=moment(x,0)+xx;                                   @ construct matrix x'x @

  if m1;                                              @ construct matrix z'Hz @
    zkz=quad1(z,t1)+zkz;
  elseif m3;
    zkz=quad3(z,t-lag-1,t1)+zkz;
  else;
    zkz=moment(z,0)+zkz;
  endif;

/*
  if mn1;                                             @ construct matrix z'Hz @
    zkz=moment(z,0)+zkz;
  else;
    zkz=quad1(z,t1)+zkz;
  endif;
*/

  if t3>=2;                                        @ serial correlation tests @
    y01=tlag(sy,0,1,t3); y10=tlag(sy,1,0,t3);
    x01=tlag(sx,0,1,t3); x10=tlag(sx,1,0,t3);
    y1y=y01'y10+y1y; x1x=x01'x10+x1x; y1x=y01'x10+y1x;
    x1y=x01'y10+x1y; clear y10,x10;
    if m0;
      yky1=moment(y01,0)+yky1;
      xkx1=moment(x01,0)+xkx1;
    else;
      yky1=quad1(y01,t3-1)+yky1;
      xkx1=quad1(x01,t3-1)+xkx1;
    endif;
    ykx1=quad(y01,x01,t3-1)+ykx1;
    clear y01,x01; nc1=nc1+mul;
    if m25;
      zky1=zkv(z,app(y,1,t3),t3)+zky1; zkx1=zkv(z,app(x,1,t3),t3)+zkx1;
    else;
      zky1=quad(z,app(y,1,t3),t3)+zky1; zkx1=quad(z,app(x,1,t3),t3)+zkx1;
    endif;
  endif;

  if t3>=3;
    y02=tlag(sy,0,2,t3); y20=tlag(sy,2,0,t3);
    x02=tlag(sx,0,2,t3); x20=tlag(sx,2,0,t3);
    y2y=y02'y20+y2y; x2x=x02'x20+x2x; y2x=y02'x20+y2x;
    x2y=x02'y20+x2y; clear y20,x20;
    if m0;
      yky2=moment(y02,0)+yky2;
      xkx2=moment(x02,0)+xkx2;
    else;
      yky2=quad1(y02,t3-2)+yky2;
      xkx2=quad1(x02,t3-2)+xkx2;
    endif;
    ykx2=quad(y02,x02,t3-2)+ykx2;
    clear y02,x02; nc2=nc2+mul;
    if m25;
      zky2=zkv(z,app(y,2,t3),t3)+zky2; zkx2=zkv(z,app(x,2,t3),t3)+zkx2;
    else;
      zky2=quad(z,app(y,2,t3),t3)+zky2; zkx2=quad(z,app(x,2,t3),t3)+zkx2;
    endif;
  endif;

  ryy=moment(ry,0)+ryy;                                                 @ RSS @
  rxx=moment(rx,0)+rxx;
  rxy=rx'ry+rxy;
  sumry=sumc(ry)+sumry;

  clear y,x,z,sy,sx,ry,rx;

  if d1;                                             @ descriptive statistics @

    if r==1;
      yxmn=minc(yx)'; yxmx=maxc(yx)';
    else;
      yxmn=yxmn|(minc(yx)'); yxmx=yxmx|(maxc(yx)');
    endif;
    mom=moment(yx,0)+mom; sumyx=sumc(yx)+sumyx;

  endif; clear yx;

  r=r+1;

endo;
@----------------------------- end of first loop -----------------------------@

rybar=sumry/obs;

if d1;                                               @ descriptive statistics @

  yxbar=sumyx./obs;
  yxmin=minc(yxmn);
  yxmax=maxc(yxmx);
  yxstd=sqrt((diag(mom)-(obs.*yxbar^2))./(obs-1));
  ayxstd=sqrt((diag(mom)-(obs.*yxbar^2))./obs);
  corr=(mom-(obs.*(yxbar*yxbar')))./(obs.*(ayxstd*ayxstd'));

endif;

@---------------------------- one-step estimates -----------------------------@
zx=xz'; inzkz=invpd(zkz); clear zkz;

xmx=xz*inzkz*zx;
xmy=xz*inzkz*zy;
inxmx=invpd(xmx); clear xmx;
b=inxmx*xmy; clear xmy;                                @ coefficient vector b @

rss=ryy+b'rxx*b-2.*b'rxy;                                               @ RSS @
tss=ryy-obs*rybar^2;                                                    @ TSS @
df=obs-k;                                                @ degrees of freedom @
s=rss./df;                                 @ sigma squared from IV regression @

vc=s*inxmx;                                               @ covariance matrix @
stderr=sqrt(diag(vc));                                      @ standard errors @
trat=b./stderr;                                                    @ t-ratios @
pvt=2*cdfnc(abs(trat));                                            @ p-values @

if m0125;

if sc2;                                            @ serial correlation tests @
  v2v=y2y+b'x2x*b-y2x*b-b'x2y;
  v2x=y2x-b'x2x;
  vhat2=(yky2+b'xkx2*b-2*ykx2*b-2*v2x*inxmx*xz*inzkz*(zky2-zkx2*b)+
         v2x*inxmx*v2x')*s;
  m2=v2v./sqrt(vhat2);
  pvm2=2*cdfnc(abs(m2));
endif;

if sc1;
  v1v=y1y+b'x1x*b-y1x*b-b'x1y;
  v1x=y1x-b'x1x;
  vhat1=(yky1+b'xkx1*b-2*ykx1*b-2*v1x*inxmx*xz*inzkz*(zky1-zkx1*b)+
         v1x*inxmx*v1x')*s;
  m1=v1v./sqrt(vhat1);
  pvm1=2*cdfnc(abs(m1));
endif;

endif;

uz=zy'-b'xz;                                                    @ Sargan test @
if iiv==1 and ijus==0;
  sar=(uz*inzkz*uz')./s; dfs=cinst-k;
  pvsar=cdfchic(sar,dfs);
endif;

if cn0;                                                          @ Wald tests @
  bc=b[2:kp,1]; vcc=vc[2:kp,2:kp];
else;
  bc=b[1:kp,1]; vcc=vc[1:kp,1:kp];
endif;
zz1=bc'invpd(vcc)*bc; dfz1=rows(bc);
pvz1=cdfchic(zz1,dfz1);

if c125;
  btim=b[kp+1:kd,1]; vctim=vc[kp+1:kd,kp+1:kd];
  if m125;
    btim=b[1 kp+1:kd,1]; vctim=vc[1 kp+1:kd,1 kp+1:kd];
  endif;
  zz2=btim'invpd(vctim)*btim; dfz2=rows(btim);
  pvz2=cdfchic(zz2,dfz2);
endif;

if c4;
  bind=b[kp+1:k,1]; vcind=vc[kp+1:k,kp+1:k];
  zz3=bind'invpd(vcind)*bind; dfz3=rows(bind);
  pvz3=cdfchic(zz3,dfz3);
endif;

if c25;
  bind=b[kd+1:k,1]; vcind=vc[kd+1:k,kd+1:k];
  zz3=bind'invpd(vcind)*bind; dfz3=rows(bind);
  pvz3=cdfchic(zz3,dfz3);
  bdum=b[kp+1:k,1]; vcdum=vc[kp+1:k,kp+1:k];
  if m125;
    bdum=b[1 kp+1:k,1]; vcdum=vc[1 kp+1:k,1 kp+1:k];
  endif;
  zz4=bdum'invpd(vcdum)*bdum; dfz4=rows(bdum);
  pvz4=cdfchic(zz4,dfz4);
endif;

if wald1;
  if cn0; testcols=testcols+1; endif;
  buser=submat(b,testcols,0); vcuser=submat(vc,testcols,testcols);
  nameus=submat(namex,0,testcols);
  zz5=buser'invpd(vcuser)*buser; dfz5=rows(buser);
  pvz5=cdfchic(zz5,dfz5);
endif;

xzinzkz=xz*inzkz;

clear inzkz,yky1,xkx1,ykx1,zky1,zkx1,yky2,xkx2,ykx2,zky2,zkx2,imike;
@------------------------- robust statistics option --------------------------@

if irob==1;

ef=seekr(f1,startf1);

clear obs,fms,hh,tt,zkz2,zomv2w,vomv2w,zomv1w,vomv1w;

cls; locate 1,1;

"    *******************   D.P.D. -  One-Step Results   *******************  ";
?;
" Var";; format 8,8; print $namex[1,cn0+1:kp]; ?;
"Coef";; format 8,3; b[cn0+1:kp,1]'; ?;
"S.E.";; stderr[cn0+1:kp,1]'; ?;
locate 14,1; clin=cline(8); locate 16,1;
"    *******************   Reading Data - Second Pass   *******************  ";
?;?;

@-------------------------------- second loop --------------------------------@

r=1;

do while r<=mm;

    gosub defdata;

    u=y-(x*b);                                              @ model residuals @
    su=sy-(sx*b);                    @ residuals for serial correlation tests @

    t2=nyears-lag-ts;                 @ maximum number of residuals for omega @

    clear y,x,sy,sx,zt;

    rs=1;                                               @ single company loop @

    do while rs<=mul;

      res=u[(rs-1)*t1+1:rs*t1,.];
      sres=su[(rs-1)*t3+1:rs*t3,.];

      org=hist[(rs-1)*t+1,1]-year1; nd=org+t3;       @ balance data for omega @

      if org>0;
        ache=zeros(org,nd)|(zeros(t3,org)~sres*sres');
        tobs=zeros(org,nd)|(zeros(t3,org)~ones(t3,t3));
      else;
        ache=sres*sres'; tobs=ones(t3,t3);
      endif;
      if (t2-nd)>0;
        ache=ache~zeros(nd,t2-nd)|zeros(t2-nd,t2);
        tobs=tobs~zeros(nd,t2-nd)|zeros(t2-nd,t2);
      endif;

      hh=hh+ache; tt=tt+tobs; clear ache,tobs;                   @ make omega @

      vv=res*res';

@         vv=vv.*eye(rows(vv));    @

      zt=z[(rs-1)*t1+1:rs*t1,.];
      zkz2=zkz2+zt'vv*zt;                                       @ make Z'vv'Z @

      svv=sres*sres';

      if mn01;                                             @ use for tests of @
        vsv=res*sres';                                @ differenced residuals @
      endif;                                          @ with other estimators @

      if t3>=3;                                    @ serial correlation tests @
        u2=su[(rs-1)*t3+1:rs*t3-2,.];
        if mn01;
          zomv2w=zomv2w+zt'vsv*(zeros(2,1)|u2);
        else;
          zomv2w=zomv2w+zt'svv*(zeros(2,1)|u2);
        endif;
        vomv2w=vomv2w+u2'svv[3:rows(svv),3:cols(svv)]*u2;
      endif;

      if t3>=2;
        u1=su[(rs-1)*t3+1:rs*t3-1,.];
        if mn01;
          zomv1w=zomv1w+zt'vsv*(zeros(1,1)|u1);
        else;
          zomv1w=zomv1w+zt'svv*(zeros(1,1)|u1);
        endif;
        vomv1w=vomv1w+u1'svv[2:rows(svv),2:cols(svv)]*u1;
      endif;

      clear u2,zt,res,vv,u1,sres,svv,vsv;

      rs=rs+1;

  endo;                                          @ end of single company loop @

  clear u,z,su;

  r=r+1;

endo;

@----------------------------- end of second loop ----------------------------@

tt=miss(tt,0);
omega=hh./tt;                                      @ serial covariance matrix @
omega1=omega./sqrt(diag(omega)*diag(omega)');     @ serial correlation matrix @

@------------------ robust statistics for one-step estimates -----------------@

xax2=xzinzkz*zkz2*xzinzkz';

vc2=inxmx*xax2*inxmx;                                     @ covariance matrix @
stderr2=sqrt(diag(vc2));                                    @ standard errors @
trat2=b./stderr2;                                                  @ t-values @
pvt2=2*cdfnc(abs(trat2));                                          @ p-values @

if sc2;                                            @ serial correlation tests @
  if m34;
    v2v=y2y+b'x2x*b-y2x*b-b'x2y;
    v2x=y2x-b'x2x;
  endif;
  vhat2w=vomv2w-(2.*v2x*inxmx*xzinzkz*zomv2w)+(v2x*vc2*v2x');
  m2w=v2v./sqrt(vhat2w);
  pvm2w=2*cdfnc(abs(m2w));
endif;

if sc1;
  if m34;
    v1v=y1y+b'x1x*b-y1x*b-b'x1y;
    v1x=y1x-b'x1x;
  endif;
  vhat1w=vomv1w-(2.*v1x*inxmx*xzinzkz*zomv1w)+(v1x*vc2*v1x');
  m1w=v1v./sqrt(vhat1w);
  pvm1w=2*cdfnc(abs(m1w));
endif;

if cinst>=fms or pseud==1; imike=1; endif;

if cn0;                                                          @ Wald tests @
  vc2c=vc2[2:kp,2:kp];
else;
  vc2c=vc2[1:kp,1:kp];
endif;
if imike==0;
  zz1w=bc'invpd(vc2c)*bc;
else;
  zz1w=bc'pinv(vc2c)*bc;
endif;
pvz1w=cdfchic(zz1w,dfz1);

if c125;
  vc2tim=vc2[kp+1:kd,kp+1:kd];
  if m125;
    vc2tim=vc2[1 kp+1:kd,1 kp+1:kd];
  endif;
  if imike==0;
    zz2w=btim'invpd(vc2tim)*btim;
  else;
    zz2w=btim'pinv(vc2tim)*btim;
  endif;
  pvz2w=cdfchic(zz2w,dfz2);
endif;

if c4;
  vc2ind=vc2[kp+1:k,kp+1:k];
  if imike==0;
    zz3w=bind'invpd(vc2ind)*bind;
  else;
    zz3w=bind'pinv(vc2ind)*bind;
  endif;
  pvz3w=cdfchic(zz3w,dfz3);
endif;

if c25;
  vc2ind=vc2[kd+1:k,kd+1:k];
  if imike==0;
    zz3w=bind'invpd(vc2ind)*bind;
  else;
    zz3w=bind'pinv(vc2ind)*bind;
  endif;
  pvz3w=cdfchic(zz3w,dfz3);
  vc2dum=vc2[kp+1:k,kp+1:k];
  if m125;
    vc2dum=vc2[1 kp+1:k,1 kp+1:k];
  endif;
  if imike==0;
    zz4w=bdum'invpd(vc2dum)*bdum;
  else;
    zz4w=bdum'pinv(vc2dum)*bdum;
  endif;
  pvz4w=cdfchic(zz4w,dfz4);
endif;

if wald1;
  vc2user=submat(vc2,testcols,testcols);
  if imike==0;
    zz5w=buser'invpd(vc2user)*buser;
  else;
    zz5w=buser'pinv(vc2user)*buser;
  endif;
  pvz5w=cdfchic(zz5w,dfz5);
endif;

@---------------------------- Two-step estimates -----------------------------@

if (iiv==1 and ijus==0);

  if imike==0;
    inzkz2=invpd(zkz2);
  else;
    inzkz2=pinv(zkz2);
  endif;
  clear zkz2;

  bxmx=xz*inzkz2*zx; bxmy=xz*inzkz2*zy;

  bvc=invpd(bxmx);                                        @ covariance matrix @

  bb=bvc*bxmy;                                           @ coefficient vector @
  bstderr=sqrt(diag(bvc));                                  @ standard errors @
  btrat=bb./bstderr;                                               @ t-values @
  bpvt=2*cdfnc(abs(btrat));                                        @ p-values @

  if sc2;                                          @ serial correlation tests @
    bv2v=y2y+bb'x2x*bb-y2x*bb-bb'x2y;
    bvhat2w=vomv2w-(2.*v2x*bvc*xz*inzkz2*zomv2w)+(v2x*bvc*v2x');
    bm2w=bv2v./sqrt(bvhat2w);
    pvbm2w=2*cdfnc(abs(bm2w));
  endif;

  if sc1;
    bv1v=y1y+bb'x1x*bb-y1x*bb-bb'x1y;
    bvhat1w=vomv1w-(2.*v1x*bvc*xz*inzkz2*zomv1w)+(v1x*bvc*v1x');
    bm1w=bv1v./sqrt(bvhat1w);
    pvbm1w=2*cdfnc(abs(bm1w));
  endif;

  buz=zy'-bb'xz; bsar=buz*inzkz2*buz';                          @ Sargan test @
  pvbsar=cdfchic(bsar,dfs);

  if cn0;                                                        @ Wald tests @
    bbc=bb[2:kp,1]; bvcc=bvc[2:kp,2:kp];
  else;
    bbc=bb[1:kp,1]; bvcc=bvc[1:kp,1:kp];
  endif;
  if imike==0;
    bzz1=bbc'invpd(bvcc)*bbc;
  else;
    bzz1=bbc'pinv(bvcc)*bbc;
  endif;
  pvbz1=cdfchic(bzz1,dfz1);

  if c125;
    bbtim=bb[kp+1:kd,1]; bvctim=bvc[kp+1:kd,kp+1:kd];
    if m125;
      bbtim=bb[1 kp+1:kd,1]; bvctim=bvc[1 kp+1:kd,1 kp+1:kd];
    endif;
    if imike==0;
      bzz2=bbtim'invpd(bvctim)*bbtim;
    else;
      bzz2=bbtim'pinv(bvctim)*bbtim;
    endif;
    pvbz2=cdfchic(bzz2,dfz2);
  endif;

  if c4;
    bbind=bb[kp+1:k,1]; bvcind=bvc[kp+1:k,kp+1:k];
    if imike==0;
      bzz3=bbind'invpd(bvcind)*bbind;
    else;
      bzz3=bbind'pinv(bvcind)*bbind;
    endif;
    pvbz3=cdfchic(bzz3,dfz3);
  endif;

  if c25;
    bbind=bb[kd+1:k,1]; bvcind=bvc[kd+1:k,kd+1:k];
    if imike==0;
      bzz3=bbind'invpd(bvcind)*bbind;
    else;
      bzz3=bbind'pinv(bvcind)*bbind;
    endif;
    pvbz3=cdfchic(bzz3,dfz3);
    bbdum=bb[kp+1:k,1]; bvcdum=bvc[kp+1:k,kp+1:k];
    if m125;
      bbdum=bb[1 kp+1:k,1]; bvcdum=bvc[1 kp+1:k,1 kp+1:k];
    endif;
    if imike==0;
      bzz4=bbdum'invpd(bvcdum)*bbdum;
    else;
      bzz4=bbdum'pinv(bvcdum)*bbdum;
    endif;
    pvbz4=cdfchic(bzz4,dfz4);
  endif;

  if wald1;
    bbuser=submat(bb,testcols,0); bvcuser=submat(bvc,testcols,testcols);
    if imike==0;
      bzz5=bbuser'invpd(bvcuser)*bbuser;
    else;
      bzz5=bbuser'pinv(bvcuser)*bbuser;
    endif;
    pvbz5=cdfchic(bzz5,dfz5);
  endif;

endif;

endif;

@------------------ end of computations - now print results ------------------@

output on;

ssd=dv+year1-1901;
im1=indmax<10;

?;?;?;?;?;?;
"  DDD     PPPP    DDD       RRRR   EEEEE   SSSS  U   U  L      TTTTT   SSSS";
"  D  D    P   P   D  D      R   R  E      S      U   U  L        T    S    ";
"  D   D   PPPP    D   D     RRRR   EEEE    SSS   U   U  L        T     SSS ";
"  D  D    P       D  D      R  R   E          S  U   U  L        T        S";
"  DDD  .  P    .  DDD  .    R   R  EEEEE  SSSS    UUU   LLLLL    T    SSSS ";
"_____________________________________________________________________________";
?;?;?;

if imod/=5;

  if iiv==0;
    "OLS,";;
  else;
    "IV,";;
  endif;

  if m0;
    " LEVELS";
  elseif imod==1;
    " FIRST DIFFERENCES";
  elseif imod==2;
    " ORTHOGONAL DEVIATIONS";
  elseif m3;
    " FIRST DIFFERENCES + LEVELS";
  elseif m4;
    " ORTHOGONAL DEVIATIONS + LEVELS";
  endif;

elseif imod==5;
  "WITHIN GROUPS";
endif;

?;?; format 7,0; "Number of firms:" fms;;
format 4,0; "     Sample period is ";; year1+lag+tl;;
                               "to ";; year1+nyears-1;
format 10,0; "Observations:"  obs;; "     Degrees of freedom:" df;

?; "Dependent variable is:";; format /m1 8,8; print $namey[1,1];
if iiv==0; goto one; endif;
?; "Instruments used are:"; ?;
print $namez; one: ?;?;

if m34;

"*****         ONE-STEP ESTIMATES WITH NON-ROBUST TEST STATISTICS        *****";
"*****         ARE NOT AVAILABLE WITH COMBINATIONS ESTIMATORS            *****";

else;

"-----------------------------------------------------------------------------";

?;?;
"                              ONE-STEP ESTIMATES";

?;?; format 14,6;  "RSS =" rss;;"          TSS =" tss;
"Estimated sigma-squared (levels)  =";; if imod==1; 0.5*s; else; s; endif; ?;

"Wald test of joint significance: " zz1;; format 4,0; "  df =" dfz1;;
format 5,3; "    p = " pvz1;

if c125; format 14,6;
"Wald test - jt sig of time dums: " zz2;; format 4,0; "  df =" dfz2;;
format 5,3; "    p = " pvz2;
endif;

if c45; format 14,6;
"Wald test - jt sig of  ind dums: " zz3;; format 4,0; "  df =" dfz3;;
format 5,3; "    p = " pvz3;
endif;

if c5; format 14,6;
"Wald test - jt sig of both dums: " zz4;; format 4,0; "  df =" dfz4;;
format 5,3; "    p = " pvz4;
endif;

if c2; format 14,6;
"Wald test - jt sig of  i/t dums: " zz3;; format 4,0; "  df =" dfz3;;
format 5,3; "    p = " pvz3;
format 14,6;
"Wald test - jt sig of  all dums: " zz4;; format 4,0; "  df =" dfz4;;
format 5,3; "    p = " pvz4;
endif;

if wald1; format 14,6; ?;
"     Wald test selected by user: " zz5;; format 4,0; "  df =" dfz5;;
format 5,3; "    p = " pvz5;
"     Testing:";; format 8,8; print $nameus;
endif;

if iiv==1 and ijus==0; format 14,6; ?;
"                    Sargan test: " sar;; format 4,0; "  df =" dfs;;
format 5,3; "    p = " pvsar;
endif;

?;
"Variable      Coefficient      Std. Error     T-Statistic        P-Value";
"------------------------------------------------------------------------";

i=1; j=1;

do until i>k;

  if i<=kp;
    format /m1 /rd 8,8;
    if k>1; print $namex[1,i];; endif;
    if k==1; print $namex[1,1];; endif;

  else;

    if c15;
      if i<=kd;
        "     D";; format 2,0; ssd[i-kp+1,1];;
      else;
        if im1; "      I";; format 1,0; dv1[i-kd+1,1];;
        else;   "     I";; format 2,0; dv1[i-kd+1,1];; endif;
      endif;

    elseif c4;
      if im1; "      I";; format 1,0; dv1[i-kp+1,1];;
      else;   "     I";; format 2,0; dv1[i-kp+1,1];; endif;

    elseif c2;
      if i<=kd;
        "     D";; format 2,0; ssd[i-kp+1,1];;
      else;
        if im1; "  I";; format 1,0; dv1[j+1,1];;
        else;   " I";; format 2,0; dv1[j+1,1];; endif;
        "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];;
        if i-kd>=j*(kd-kp+1); j=j+1; endif;
      endif;

    endif;

  endif;

  format 15,6; b[i,1];; stderr[i,1];; trat[i,1];; /rd pvt[i,1];

i=i+1;

endo;

if sc1;
  ?;format 8,3;
  "Test for  first-order serial correlation:";; m1;; " [";; format 5,0; nc1;;
  "]";; format 5,3; "    p = " pvm1;
endif;

if sc2;
  format 8,3;
  "Test for second-order serial correlation:";; m2;; " [";; format 5,0; nc2;;
  "]";; format 5,3; "    p = " pvm2;
endif;

?;

"NOTE: Standard errors and test statistics not robust to heteroskedasticity";

endif;

if irob==1; ?;?;

"-----------------------------------------------------------------------------";

?;?;
"                ONE-STEP ESTIMATES WITH ROBUST TEST STATISTICS";
?;?; format 14,6;

if m34;
  "RSS =" rss;;"          TSS =" tss;
  "Estimated sigma-squared (levels)  =";; s; ?;
endif;

"Wald test of joint significance: " zz1w;; format 4,0; "  df =" dfz1;;
format 5,3; "    p = " pvz1w;

if c125; format 14,6;
"Wald test - jt sig of time dums: " zz2w;; format 4,0; "  df =" dfz2;;
format 5,3; "    p = " pvz2w;
endif;

if c45; format 14,6;
"Wald test - jt sig of  ind dums: " zz3w;; format 4,0; "  df =" dfz3;;
format 5,3; "    p = " pvz3w;
endif;

if c5; format 14,6;
"Wald test - jt sig of both dums: " zz4w;; format 4,0; "  df =" dfz4;;
format 5,3; "    p = " pvz4w;
endif;

if c2; format 14,6;
"Wald test - jt sig of  i/t dums: " zz3w;; format 4,0; "  df =" dfz3;;
format 5,3; "    p = " pvz3w;
format 14,6;
"Wald test - jt sig of  all dums: " zz4w;; format 4,0; "  df =" dfz4;;
format 5,3; "    p = " pvz4w;
endif;

if wald1; format 14,6; ?;
"     Wald test selected by user: " zz5w;; format 4,0; "  df =" dfz5;;
format 5,3; "    p = " pvz5w;
"     Testing:";; format 8,8; print $nameus;
endif;

?;
"Variable      Coefficient      Std. Error     T-Statistic        P-Value";
"------------------------------------------------------------------------";

i=1; j=1;

do until i>k;

  if i<=kp;
    format /m1 /rd 8,8;
    if k>1; print $namex[1,i];; endif;
    if k==1; print $namex[1,1];; endif;

  else;

    if c15;
      if i<=kd;
        "     D";; format 2,0; ssd[i-kp+1,1];;
      else;
        if im1; "      I";; format 1,0; dv1[i-kd+1,1];;
        else;   "     I";; format 2,0; dv1[i-kd+1,1];; endif;
      endif;

    elseif c4;
      if im1; "      I";; format 1,0; dv1[i-kp+1,1];;
      else;   "     I";; format 2,0; dv1[i-kp+1,1];; endif;

    elseif c2;
      if i<=kd;
        "     D";; format 2,0; ssd[i-kp+1,1];;
      else;
        if im1; "  I";; format 1,0; dv1[j+1,1];;
        else;   " I";; format 2,0; dv1[j+1,1];; endif;
        "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];;
        if i-kd>=j*(kd-kp+1); j=j+1; endif;
      endif;

    endif;

  endif;

  format 15,6; b[i,1];; stderr2[i,1];; trat2[i,1];; /rd pvt2[i,1];

i=i+1;

endo;

if sc1;
  ?;format 8,3;
  "Test for  first-order serial correlation:";; m1w;; " [";; format 5,0; nc1;;
  "]";; format 5,3; "    p = " pvm1w;
endif;

if sc2;
  format 8,3;
  "Test for second-order serial correlation:";; m2w;; " [";; format 5,0; nc2;;
  "]";; format 5,3; "    p = " pvm2w;
endif;

?;?; outwidth 252;
"          Estimated serial correlation matrix - ";;
if m0;
  "levels residuals";
else;
  "differenced residuals";
endif;

?;
i=1;
do while i<=cols(omega1);
  j=1;
  do while j<=i; format 6,3;
    if j==i;
      omega1[j,i];
    else;
      omega1[j,i];;
    endif;
  j=j+1;
  endo;
i=i+1;
endo;

?;
"          Number of observations available to sample covariances";
?;
i=1;
do while i<=cols(tt);
  j=1;
  do while j<=i; format 6,0;
    if j==i;
      tt[j,i];
    else;
      tt[j,i];;
    endif;
  j=j+1;
  endo;
i=i+1;
endo;

outwidth 80;
if (iiv==1 and ijus==0);

?;?;
"-----------------------------------------------------------------------------";
?;?;
"                              TWO-STEP ESTIMATES";

?;?; format 14,6;

"Wald test of joint significance: " bzz1;; format 4,0; "  df =" dfz1;;
format 5,3; "    p = " pvbz1;

if c125; format 14,6;
"Wald test - jt sig of time dums: " bzz2;; format 4,0; "  df =" dfz2;;
format 5,3; "    p = " pvbz2;
endif;

if c45; format 14,6;
"Wald test - jt sig of  ind dums: " bzz3;; format 4,0; "  df =" dfz3;;
format 5,3; "    p = " pvbz3;
endif;

if c5; format 14,6;
"Wald test - jt sig of both dums: " bzz4;; format 4,0; "  df =" dfz4;;
format 5,3; "    p = " pvbz4;
endif;

if c2; format 14,6;
"Wald test - jt sig of  i/t dums: " bzz3;; format 4,0; "  df =" dfz3;;
format 5,3; "    p = " pvbz3;
format 14,6;
"Wald test - jt sig of  all dums: " bzz4;; format 4,0; "  df =" dfz4;;
format 5,3; "    p = " pvbz4;
endif;

if wald1; format 14,6; ?;
"     Wald test selected by user: " bzz5;; format 4,0; "  df =" dfz5;;
format 5,3; "    p = " pvbz5;
"     Testing:";; format 8,8; print $nameus;
endif;

format 14,6; ?;
"                    Sargan test: " bsar;; format 4,0; "  df =" dfs;;
format 5,3; "    p = " pvbsar;

?;
"Variable      Coefficient      Std. Error     T-Statistic        P-Value";
"------------------------------------------------------------------------";

i=1; j=1;

do until i>k;

  if i<=kp;
    format /m1 /rd 8,8;
    if k>1; print $namex[1,i];; endif;
    if k==1; print $namex[1,1];; endif;

  else;

    if c15;
      if i<=kd;
        "     D";; format 2,0; ssd[i-kp+1,1];;
      else;
        if im1; "      I";; format 1,0; dv1[i-kd+1,1];;
        else;   "     I";; format 2,0; dv1[i-kd+1,1];; endif;
      endif;

    elseif c4;
      if im1; "      I";; format 1,0; dv1[i-kp+1,1];;
      else;   "     I";; format 2,0; dv1[i-kp+1,1];; endif;

    elseif c2;
      if i<=kd;
        "     D";; format 2,0; ssd[i-kp+1,1];;
      else;
        if im1; "  I";; format 1,0; dv1[j+1,1];;
        else;   " I";; format 2,0; dv1[j+1,1];; endif;
        "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];;
        if i-kd>=j*(kd-kp+1); j=j+1; endif;
      endif;

    endif;

  endif;

  format 15,6; bb[i,1];; bstderr[i,1];; btrat[i,1];; /rd bpvt[i,1];

i=i+1;

endo;

if sc1;
  ?;format 8,3;
  "Test for  first-order serial correlation:";; bm1w;; " [";; format 5,0; nc1;;
  "]";; format 5,3; "    p = " pvbm1w;
endif;

if sc2;
  format 8,3;
  "Test for second-order serial correlation:";; bm2w;; " [";; format 5,0; nc2;;
  "]";; format 5,3; "    p = " pvbm2w;
endif;

if cinst>=fms;
?;?;
"NOTE: Generalised inverses were used to compute two-step estimates";
"      with more instruments than cross-section units";
endif;

if pseud==1;
?;?;
"NOTE: Generalised inverses were used to compute two-step estimates";
"      at the user's request";
endif;

elseif ijus==1;

  ?;?;
  "MODEL JUST IDENTIFIED - TWO-STEP ESTIMATES AND ONE-STEP ESTIMATES COINCIDE";

elseif iiv==0;

  if imod/=5;
    ?;?;
    "OLS ESTIMATION - TWO-STEP ESTIMATES AND ONE-STEP ESTIMATES COINCIDE";
  elseif imod==5;
    ?;?;
    "WITHIN GROUPS - TWO-STEP ESTIMATES AND ONE-STEP ESTIMATES COINCIDE";
  endif;

endif;

endif;

outwidth 252;
if d1;

?;?;
"-----------------------------------------------------------------------------";
?;?;

if imod==0 or imod==3 or imod==4;
  "                DESCRIPTIVE STATISTICS - VARIABLES IN LEVELS";
elseif m25;
  "         DESCRIPTIVE STATISTICS - VARIABLES IN ORTHOGONAL DEVIATIONS";
elseif imod==1;
  "           DESCRIPTIVE STATISTICS - VARIABLES IN FIRST DIFFERENCES";
endif;

?;?;
"Variable        Mean         Std Dev         Min           Max";
"----------------------------------------------------------------";

format /m1 /rd 8,8; print $namey[1,1];;
format 13,5; yxbar[1,1];; yxstd[1,1];; yxmin[1,1];; yxmax[1,1];

i=1;
do while i<=kk-1;
  format /m1 /rd 8,8; print $nameyx[1,i];;
  format 13,5; yxbar[i+1,1];; yxstd[i+1,1];; yxmin[i+1,1];; yxmax[i+1,1];
i=i+1;
endo;

?;?;
"Correlation Matrix";

?;?;
format /m1 /rd 8,8; print $namey[1,1];;

i=1;
do while i<=kk-1;
  format /m1 /rd 8,8; print $nameyx[1,i];;
i=i+1;
endo;

?;?; format /m1 8,2;
i=1;
do while i<=kk;
  j=1;
  do while j<=i;
    if j==i;
      corr[j,i];
    else;
      corr[j,i];;
    endif;
  j=j+1;
  endo;
i=i+1;
endo;

endif;

if icov==1;

?;?;
"-----------------------------------------------------------------------------";
?;?;
"                         ASYMPTOTIC VARIANCE MATRICES";

if m0125;

?;?;
"Non-robust AVM of one-step estimates (x100000)"; ?;?;
vcp=vc.*100000;

i=1; j=1;
do until i>k;
  if i<=kp; format /m1 /rd 8,8;
    if i==k;
      print $namex[1,i];
    else;
      print $namex[1,i];;
    endif;
  else;
    if c15;
      if i<=kd; format 2,0;
        if i==k;
          "     D";; ssd[i-kp+1,1];
        else;
          "     D";; ssd[i-kp+1,1];;
        endif;
      else;
        if i==k;
          if im1; "      I";; format 1,0; dv1[i-kd+1,1];
          else;   "     I";; format 2,0; dv1[i-kd+1,1]; endif;
        else;
          if im1; "      I";; format 1,0; dv1[i-kd+1,1];;
          else;   "     I";; format 2,0; dv1[i-kd+1,1];; endif;
        endif;
      endif;
    elseif c4;
      if i==k;
        if im1; "      I";; format 1,0; dv1[i-kp+1,1];
        else;   "     I";; format 2,0; dv1[i-kp+1,1]; endif;
      else;
        if im1; "      I";; format 1,0; dv1[i-kp+1,1];;
        else;   "     I";; format 2,0; dv1[i-kp+1,1];; endif;
      endif;
    elseif c2;
      if i<=kd; format 2,0;
        if i==k;
          "     D";; ssd[i-kp+1,1];
        else;
          "     D";; ssd[i-kp+1,1];;
        endif;
      else;
        if i==k;
          if im1; "  I";; format 1,0; dv1[j+1,1];;
          else;   " I";; format 2,0; dv1[j+1,1];; endif;
          "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];
        else;
          if im1; "  I";; format 1,0; dv1[j+1,1];;
          else;   " I";; format 2,0; dv1[j+1,1];; endif;
          "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];;
        endif;
        if i-kd>=j*(kd-kp+1); j=j+1; endif;
      endif;
    endif;
  endif;
i=i+1;
endo;

?;
i=1;
do while i<=cols(vc);
  j=1;
  do while j<=i; format 8,3;
    if j==i;
      vcp[j,i];
    else;
      vcp[j,i];;
    endif;
  j=j+1;
  endo;
i=i+1;
endo;

endif;

if irob==1;

?;?;
"Robust AVM of one-step estimates (x100000)"; ?;?;
vcp2=vc2.*100000;

i=1; j=1;
do until i>k;
  if i<=kp; format /m1 /rd 8,8;
    if i==k;
      print $namex[1,i];
    else;
      print $namex[1,i];;
    endif;
  else;
    if c15;
      if i<=kd; format 2,0;
        if i==k;
          "     D";; ssd[i-kp+1,1];
        else;
          "     D";; ssd[i-kp+1,1];;
        endif;
      else;
        if i==k;
          if im1; "      I";; format 1,0; dv1[i-kd+1,1];
          else;   "     I";; format 2,0; dv1[i-kd+1,1]; endif;
        else;
          if im1; "      I";; format 1,0; dv1[i-kd+1,1];;
          else;   "     I";; format 2,0; dv1[i-kd+1,1];; endif;
        endif;
      endif;
    elseif c4;
      if i==k;
        if im1; "      I";; format 1,0; dv1[i-kp+1,1];
        else;   "     I";; format 2,0; dv1[i-kp+1,1]; endif;
      else;
        if im1; "      I";; format 1,0; dv1[i-kp+1,1];;
        else;   "     I";; format 2,0; dv1[i-kp+1,1];; endif;
      endif;
    elseif c2;
      if i<=kd; format 2,0;
        if i==k;
          "     D";; ssd[i-kp+1,1];
        else;
          "     D";; ssd[i-kp+1,1];;
        endif;
      else;
        if i==k;
          if im1; "  I";; format 1,0; dv1[j+1,1];;
          else;   " I";; format 2,0; dv1[j+1,1];; endif;
          "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];
        else;
          if im1; "  I";; format 1,0; dv1[j+1,1];;
          else;   " I";; format 2,0; dv1[j+1,1];; endif;
          "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];;
        endif;
        if i-kd>=j*(kd-kp+1); j=j+1; endif;
      endif;
    endif;
  endif;
i=i+1;
endo;

?;
i=1;
do while i<=cols(vc2);
  j=1;
  do while j<=i; format 8,3;
    if j==i;
      vcp2[j,i];
    else;
      vcp2[j,i];;
    endif;
  j=j+1;
  endo;
i=i+1;
endo;

if iiv==1 and ijus==0;

?;?;
"Robust AVM of two-step estimates (x100000)"; ?;?;
bvcp=bvc.*100000;


i=1; j=1;
do until i>k;
  if i<=kp; format /m1 /rd 8,8;
    if i==k;
      print $namex[1,i];
    else;
      print $namex[1,i];;
    endif;
  else;
    if c15;
      if i<=kd; format 2,0;
        if i==k;
          "     D";; ssd[i-kp+1,1];
        else;
          "     D";; ssd[i-kp+1,1];;
        endif;
      else;
        if i==k;
          if im1; "      I";; format 1,0; dv1[i-kd+1,1];
          else;   "     I";; format 2,0; dv1[i-kd+1,1]; endif;
        else;
          if im1; "      I";; format 1,0; dv1[i-kd+1,1];;
          else;   "     I";; format 2,0; dv1[i-kd+1,1];; endif;
        endif;
      endif;
    elseif c4;
      if i==k;
        if im1; "      I";; format 1,0; dv1[i-kp+1,1];
        else;   "     I";; format 2,0; dv1[i-kp+1,1]; endif;
      else;
        if im1; "      I";; format 1,0; dv1[i-kp+1,1];;
        else;   "     I";; format 2,0; dv1[i-kp+1,1];; endif;
      endif;
    elseif c2;
      if i<=kd; format 2,0;
        if i==k;
          "     D";; ssd[i-kp+1,1];
        else;
          "     D";; ssd[i-kp+1,1];;
        endif;
      else;
        if i==k;
          if im1; "  I";; format 1,0; dv1[j+1,1];;
          else;   " I";; format 2,0; dv1[j+1,1];; endif;
          "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];
        else;
          if im1; "  I";; format 1,0; dv1[j+1,1];;
          else;   " I";; format 2,0; dv1[j+1,1];; endif;
          "D";; format 2,0; ssd[(i-kd)-(j-1)*(kd-kp+1),1];;
        endif;
        if i-kd>=j*(kd-kp+1); j=j+1; endif;
      endif;
    endif;
  endif;
i=i+1;
endo;

?;
i=1;
do while i<=cols(bvc);
  j=1;
  do while j<=i; format 8,3;
    if j==i;
      bvcp[j,i];
    else;
      bvcp[j,i];;
    endif;
  j=j+1;
  endo;
i=i+1;
endo;

endif;

endif;

endif;

if isav==1;

  save beta1=b;
  if m0125;
    save var=vc;
  endif;
  if irob==1;
    save var1=vc2;
    if iiv==1 and ijus==0;
      save beta2=bb, var2=bvc;
    endif;
  endif;
  ?; ?;
  "  Parameter estimates and covariance matrices saved to GAUSS matrices";

endif;

?;?; format 10,3; outwidth 80;
"  Execution time is ";; cts=hsec-speed; cts=cts/100;
cts;;" seconds ";

if sys==1; system; endif;
@------------------------------ End of program -------------------------------@

